@@ -8,7 +8,6 @@ from time import sleep
 import select
 import socket
 from tornado import gen
-import time
 
 import zmq
 from zmq.eventloop import ioloop
@@ -18,7 +17,8 @@ from circus.exc import AlreadyExist
 from circus import logger
 from circus.watcher import Watcher
 from circus.util import debuglog, _setproctitle, parse_env_dict
-from circus.util import DictDiffer, synchronized, tornado_sleep
+from circus.util import DictDiffer, synchronized, tornado_sleep, papa
+from circus.util import IS_WINDOWS
 from circus.config import get_config
 from circus.plugins import get_plugin_cmd
 from circus.sockets import CircusSocket, CircusSockets
@@ -75,6 +75,7 @@ class Arbiter(object):
     - **fqdn_prefix** -- a prefix for the unique identifier of the circus
                          instance on the cluster.
     - **endpoint_owner** -- unix user to chown the endpoint to if using ipc.
+    - **papa_endpoint** -- the papa process kernel endpoint
     """
 
     def __init__(self, watchers, endpoint, pubsub_endpoint, check_delay=1.0,
@@ -86,7 +87,8 @@ class Arbiter(object):
                  httpd_close_outputs=False, debug=False, debug_gc=False,
                  ssh_server=None, proc_name='circusd', pidfile=None,
                  loglevel=None, logoutput=None, loggerconfig=None,
-                 fqdn_prefix=None, umask=None, endpoint_owner=None):
+                 fqdn_prefix=None, umask=None, endpoint_owner=None,
+                 papa_endpoint=None):
 
         self.watchers = watchers
         self.endpoint = endpoint
@@ -116,6 +118,16 @@ class Arbiter(object):
             fqdn = '{}@{}'.format(fqdn_prefix, socket_fqdn)
         self.fqdn = fqdn
 
+        if papa_endpoint and papa:
+            if papa_endpoint.startswith('ipc:/'):
+                papa_endpoint = papa_endpoint[4:]
+                while papa_endpoint[:2] == '//':
+                    papa_endpoint = papa_endpoint[1:]
+                papa.set_default_path(papa_endpoint)
+            elif papa_endpoint.startswith('tcp://'):
+                papa_endpoint = papa_endpoint[6:].partition(':')[2]
+                papa.set_default_port = papa_endpoint
+
         self.ctrl = self.loop = None
         self._provided_loop = False
         self.socket_event = False
@@ -495,7 +507,7 @@ class Arbiter(object):
 
     @gen.coroutine
     @debuglog
-    def start(self):
+    def start(self, cb=None):
         """Starts all the watchers.
 
         If the ioloop has been provided during __init__() call,
@@ -505,6 +517,10 @@ class Arbiter(object):
         starts all watchers and the eventloop (and blocks here). In this mode
         the method MUST NOT yield anything because it's called as a standard
         method.
+
+        :param cb: Callback called after all the watchers have been started,
+                   when the loop hasn't been provided.
+        :type function:
         """
         logger.info("Starting master on pid %s", self.pid)
         self.initialize()
@@ -519,7 +535,9 @@ class Arbiter(object):
                 yield self.start_watchers()
             else:
                 # start_watchers will be called just after the start_io_loop()
-                self.loop.add_future(self.start_watchers(), lambda x: None)
+                if not cb:
+                    def cb(x): pass
+                self.loop.add_future(self.start_watchers(), cb)
             logger.info('Arbiter now waiting for commands')
             self._running = True
             if not self._provided_loop:
@@ -557,7 +575,7 @@ class Arbiter(object):
     @synchronized("arbiter_stop")
     @gen.coroutine
     def stop(self):
-        yield self._stop()
+        yield self._stop(True)
 
     @gen.coroutine
     def _emergency_stop(self):
@@ -569,20 +587,18 @@ class Arbiter(object):
         self.stop_controller_and_close_sockets()
 
     @gen.coroutine
-    def _stop(self):
+    def _stop(self, for_shutdown=False):
         logger.info('Arbiter exiting')
         self._stopping = True
-        yield self._stop_watchers(close_output_streams=True)
+        yield self._stop_watchers(close_output_streams=True,
+                                  for_shutdown=for_shutdown)
         if self._provided_loop:
             cb = self.stop_controller_and_close_sockets
             self.loop.add_callback(cb)
         else:
-            self.loop.add_timeout(time.time() + 1, self._stop_cb)
-
-    def _stop_cb(self):
-        self.loop.stop()
-        # stop_controller_and_close_sockets will be
-        # called in the end of start() method
+            # stop_controller_and_close_sockets will be
+            # called in the end of start() method
+            self.loop.add_callback(self.loop.stop)
 
     def reap_processes(self):
         # map watcher to pids
@@ -593,25 +609,26 @@ class Arbiter(object):
                     watchers_pids[process.pid] = watcher
 
         # detect dead children
-        while True:
-            try:
-                # wait for our child (so it's not a zombie)
-                pid, status = os.waitpid(-1, os.WNOHANG)
-                if not pid:
-                    break
+        if not IS_WINDOWS:
+            while True:
+                try:
+                    # wait for our child (so it's not a zombie)
+                    pid, status = os.waitpid(-1, os.WNOHANG)
+                    if not pid:
+                        break
 
-                if pid in watchers_pids:
-                    watcher = watchers_pids[pid]
-                    watcher.reap_process(pid, status)
-            except OSError as e:
-                if e.errno == errno.EAGAIN:
-                    sleep(0)
-                    continue
-                elif e.errno == errno.ECHILD:
-                    # process already reaped
-                    return
-                else:
-                    raise
+                    if pid in watchers_pids:
+                        watcher = watchers_pids[pid]
+                        watcher.reap_process(pid, status)
+                except OSError as e:
+                    if e.errno == errno.EAGAIN:
+                        sleep(0)
+                        continue
+                    elif e.errno == errno.ECHILD:
+                        # process already reaped
+                        return
+                    else:
+                        raise
 
     @synchronized("manage_watchers")
     @gen.coroutine
@@ -675,7 +692,7 @@ class Arbiter(object):
 
     def get_watcher(self, name):
         """Return the watcher *name*."""
-        return self._watchers_names[name]
+        return self._watchers_names[name.lower()]
 
     def statuses(self):
         return dict([(watcher.name, watcher.status())
@@ -706,7 +723,7 @@ class Arbiter(object):
 
     @synchronized("arbiter_rm_watcher")
     @gen.coroutine
-    def rm_watcher(self, name):
+    def rm_watcher(self, name, nostop=False):
         """Deletes a watcher.
 
         Options:
@@ -716,48 +733,59 @@ class Arbiter(object):
         logger.debug('Deleting %r watcher', name)
 
         # remove the watcher from the list
-        watcher = self._watchers_names.pop(name)
+        watcher = self._watchers_names.pop(name.lower())
         del self.watchers[self.watchers.index(watcher)]
 
-        # stop the watcher
-        yield watcher._stop()
+        if not nostop:
+            # stop the watcher
+            yield watcher._stop()
 
     @synchronized("arbiter_start_watchers")
     @gen.coroutine
-    def start_watchers(self):
-        yield self._start_watchers()
+    def start_watchers(self, watcher_iter_func=None):
+        yield self._start_watchers(watcher_iter_func=watcher_iter_func)
 
     @gen.coroutine
-    def _start_watchers(self):
-        for watcher in self.iter_watchers():
+    def _start_watchers(self, watcher_iter_func=None):
+        if watcher_iter_func is None:
+            watchers = self.iter_watchers()
+        else:
+            watchers = watcher_iter_func()
+        for watcher in watchers:
             if watcher.autostart:
                 yield watcher._start()
                 yield tornado_sleep(self.warmup_delay)
 
     @gen.coroutine
     @debuglog
-    def _stop_watchers(self, close_output_streams=False):
-        yield [w._stop(close_output_streams)
-               for w in self.iter_watchers(reverse=False)]
+    def _stop_watchers(self, close_output_streams=False,
+                       watcher_iter_func=None, for_shutdown=False):
+        if watcher_iter_func is None:
+            watchers = self.iter_watchers(reverse=False)
+        else:
+            watchers = watcher_iter_func(reverse=False)
+        yield [w._stop(close_output_streams, for_shutdown)
+               for w in watchers]
 
     @synchronized("arbiter_stop_watchers")
     @gen.coroutine
-    def stop_watchers(self):
-        yield self._stop_watchers()
+    def stop_watchers(self, watcher_iter_func=None):
+        yield self._stop_watchers(watcher_iter_func=watcher_iter_func)
 
     @gen.coroutine
-    def _restart(self, inside_circusd=False):
+    def _restart(self, inside_circusd=False, watcher_iter_func=None):
         if inside_circusd:
             self._restarting = True
             yield self._stop()
         else:
-            yield self._stop_watchers()
-            yield self._start_watchers()
+            yield self._stop_watchers(watcher_iter_func=watcher_iter_func)
+            yield self._start_watchers(watcher_iter_func=watcher_iter_func)
 
     @synchronized("arbiter_restart")
     @gen.coroutine
-    def restart(self, inside_circusd=False):
-        yield self._restart(inside_circusd=inside_circusd)
+    def restart(self, inside_circusd=False, watcher_iter_func=None):
+        yield self._restart(inside_circusd=inside_circusd,
+                            watcher_iter_func=watcher_iter_func)
 
     @property
     def endpoint_owner_mode(self):
