@@ -34,17 +34,36 @@ class ListSockets(Command):
         return self.make_message()
 
     def execute(self, arbiter, props):
-        sockets = [{'fd': socket.fileno(), 'name': socket.name,
-                    'host': socket.host, 'port': socket.port,
+
+        def _get_info(socket):
+            sock = {'fd': socket.fileno(),
+                    'name': socket.name,
                     'backlog': socket.backlog}
-                   for socket in arbiter.sockets.values()]
+
+            if socket.host is not None:
+                sock['host'] = socket.host
+                sock['port'] = socket.port
+            else:
+                sock['path'] = socket.path
+
+            return sock
+
+        sockets = [_get_info(socket) for socket in arbiter.sockets.values()]
         sockets.sort(key=operator.itemgetter('fd'))
         return {"sockets": sockets}
 
     def console_msg(self, msg):
         if 'sockets' in msg:
-            return "\n".join(['%d:socket %r at %s:%d' % (
-                s['fd'], s['name'], s['host'], s['port'])
-                for s in msg['sockets']])
+            sockets = []
+            for sock in msg['sockets']:
+                d = "%(fd)d:socket '%(name)s' "
+                if 'path' in sock:
+                    d = (d + 'at %(path)s') % sock
+                else:
+                    d = (d + 'at %(host)s:%(port)d') % sock
+
+                sockets.append(d)
+
+            return "\n".join(sockets)
 
         return self.console_error(msg)
