@@ -1,9 +1,11 @@
 from circus.exc import ArgumentError, MessageError
 from circus.py3compat import string_types
 from circus import util
-import resource
 import warnings
-
+try:
+    import resource
+except ImportError:
+    resource = None     # NOQA
 
 _HOOKS = ('before_start', 'after_start', 'before_stop', 'after_stop',
           'before_spawn', 'after_spawn', 'before_signal', 'after_signal',
@@ -51,6 +53,10 @@ def convert_option(key, val):
         return util.to_bool(val)
     elif key == "singleton":
         return util.to_bool(val)
+    elif key == "close_child_stdout":
+        return util.to_bool(val)
+    elif key == "close_child_stderr":
+        return util.to_bool(val)
     elif key.startswith('stderr_stream.') or key.startswith('stdout_stream.'):
         subkey = key.split('.', 1)[-1]
         if subkey in ('max_bytes', 'backup_count'):
@@ -90,7 +96,8 @@ def validate_option(key, val):
                   'shell', 'env', 'cmd', 'args', 'copy_env', 'retry_in',
                   'max_retry', 'graceful_timeout', 'stdout_stream',
                   'stderr_stream', 'max_age', 'max_age_variance', 'respawn',
-                  'singleton', 'hooks')
+                  'singleton', 'hooks', 'close_child_stdout',
+                  'close_child_stderr')
 
     valid_prefixes = ('stdout_stream.', 'stderr_stream.', 'hooks.', 'rlimit_')
 
@@ -116,7 +123,8 @@ def validate_option(key, val):
         if not isinstance(val, int) and not isinstance(val, string_types):
             raise MessageError("%r isn't an integer or string" % key)
 
-    elif key in ('send_hup', 'shell', 'copy_env', 'respawn', 'stop_children'):
+    elif key in ('send_hup', 'shell', 'copy_env', 'respawn', 'stop_children',
+                 'close_child_stdout', 'close_child_stderr'):
         if not isinstance(val, bool):
             raise MessageError("%r isn't a valid boolean" % key)
 
@@ -146,9 +154,17 @@ def validate_option(key, val):
                           "anymore for %r" % key)
 
     elif key.startswith('rlimit_'):
-        rlimit_key = key[7:]
-        rlimit_int = getattr(resource, 'RLIMIT_' + rlimit_key.upper(), None)
-        if rlimit_int is None:
-            raise MessageError("%r isn't a valid rlimit setting" % key)
-        if not isinstance(val, int):
+        if resource:
+            rlimit_key = key[7:]
+            rlimit_int = getattr(
+                resource, 'RLIMIT_' + rlimit_key.upper(), None
+            )
+            if rlimit_int is None:
+                raise MessageError("%r isn't a valid rlimit setting" % key)
+        else:
+            raise MessageError("rlimit options are not supported on this"
+                               " platform")
+
+        # note that a null val means RLIM_INFINITY
+        if val is not None and not isinstance(val, int):
             raise MessageError("%r rlimit value isn't a valid int" % val)
