@@ -18,3 +18,9 @@ class ArgumentError(Exception):
     """Exception raised when one argument or the number of
     arguments is invalid"""
     pass
+
+
+class ConflictError(Exception):
+    """Exception raised when one exclusive command is already running
+    in background"""
+    pass
