@@ -1,7 +1,9 @@
 import socket
+
 from zmq.eventloop import ioloop
-from zmq.utils.strtypes import u
+
 from circus.plugins import CircusPlugin
+from circus.util import human2bytes
 
 
 class StatsdClient(object):
@@ -62,11 +64,12 @@ class StatsdEmitter(CircusPlugin):
                                        config.get('sample_rate', '1.0')))
 
     def handle_recv(self, data):
-        topic, msg = data
-        topic_parts = u(topic).split(".")
-        watcher = topic_parts[1]
-        action = topic_parts[2]
-        self.statsd.increment('%s.%s' % (watcher, action))
+        watcher_name, action, msg = self.split_data(data)
+        self.statsd.increment('%s.%s' % (watcher_name, action))
+
+    def stop(self):
+        self.statsd.stop()
+        super(StatsdEmitter, self).stop()
 
 
 class BaseObserver(StatsdEmitter):
@@ -108,11 +111,13 @@ class FullStats(BaseObserver):
 
             cpus = []
             mems = []
+            mem_infos = []
 
             for sub_name, sub_info in stats.items():
                 if isinstance(sub_info, dict):
                     cpus.append(sub_info['cpu'])
                     mems.append(sub_info['mem'])
+                    mem_infos.append(human2bytes(sub_info['mem_info1']))
                 elif sub_name == "spawn_count":
                     # spawn_count info is in the same level as processes
                     # dict infos, so if spawn_count is given, take it and
@@ -126,7 +131,10 @@ class FullStats(BaseObserver):
                 # if there are only dead processes, we have an empty list
                 # and we can't measure it
                 continue
+
             self.statsd.gauge("_stats.%s.cpu_max" % name, max(cpus))
             self.statsd.gauge("_stats.%s.cpu_sum" % name, sum(cpus))
-            self.statsd.gauge("_stats.%s.mem_max" % name, max(mems))
-            self.statsd.gauge("_stats.%s.mem_sum" % name, sum(mems))
+            self.statsd.gauge("_stats.%s.mem_pct_max" % name, max(mems))
+            self.statsd.gauge("_stats.%s.mem_pct_sum" % name, sum(mems))
+            self.statsd.gauge("_stats.%s.mem_max" % name, max(mem_infos))
+            self.statsd.gauge("_stats.%s.mem_sum" % name, sum(mem_infos))
