@@ -11,19 +11,23 @@ except ImportError:
 import sys
 import errno
 import os
-import resource
 from subprocess import PIPE
 import time
 import shlex
 import warnings
+try:
+    import resource
+except ImportError:
+    resource = None     # NOQA
 
-from psutil import Popen, STATUS_ZOMBIE, STATUS_DEAD, NoSuchProcess
+from psutil import (Popen, STATUS_ZOMBIE, STATUS_DEAD, NoSuchProcess,
+                    AccessDenied)
 
 from circus.py3compat import bytestring, string_types, quote
 from circus.sockets import CircusSocket
 from circus.util import (get_info, to_uid, to_gid, debuglog, get_working_dir,
-                         ObjectDict, replace_gnu_args, is_win, get_default_gid,
-                         get_username_from_uid)
+                         ObjectDict, replace_gnu_args, get_default_gid,
+                         get_username_from_uid, IS_WINDOWS)
 from circus import logger
 
 
@@ -149,7 +153,8 @@ class Process(object):
     - **rlimits**: a mapping containing rlimit names and values that will
       be set before the command runs.
 
-    - **use_fds**: if True, will not close the fds in the subprocess.
+    - **use_fds**: if True, will not close the fds in the subprocess. Must be
+      be set to True on Windows if stdout or stderr are redirected.
       default: False.
 
     - **pipe_stdout**: if True, will open a PIPE on stdout. default: True.
@@ -162,12 +167,13 @@ class Process(object):
     - **close_child_stderr**: If True, redirects the child process' stdout
       to /dev/null after the fork. default: False.
     """
-    def __init__(self, wid, cmd, args=None, working_dir=None, shell=False,
-                 uid=None, gid=None, env=None, rlimits=None, executable=None,
-                 use_fds=False, watcher=None, spawn=True,
+    def __init__(self, name, wid, cmd, args=None, working_dir=None,
+                 shell=False, uid=None, gid=None, env=None, rlimits=None,
+                 executable=None, use_fds=False, watcher=None, spawn=True,
                  pipe_stdout=True, pipe_stderr=True,
                  close_child_stdout=False, close_child_stderr=False):
 
+        self.name = name
         self.wid = wid
         self.cmd = cmd
         self.args = args
@@ -192,10 +198,18 @@ class Process(object):
         self.stopping = False
         # sockets created before fork, should be let go after.
         self._sockets = []
+        self._worker = None
+        self.redirected = False
+        self.started = 0
 
         if self.uid is not None and self.gid is None:
             self.gid = get_default_gid(self.uid)
 
+        if IS_WINDOWS:
+            if not self.use_fds and (self.pipe_stderr or self.pipe_stdout):
+                raise ValueError("On Windows, you can't close the fds if "
+                                 "you are redirecting stdout or stderr")
+
         if spawn:
             self.spawn()
 
@@ -243,11 +257,12 @@ class Process(object):
         return sockets_fds
 
     def spawn(self):
+        self.started = time.time()
         sockets_fds = self._get_sockets_fds()
 
         args = self.format_args(sockets_fds=sockets_fds)
 
-        def preexec_fn():
+        def preexec():
             streams = [sys.stdin]
 
             if self.close_child_stdout:
@@ -259,12 +274,41 @@ class Process(object):
             self._null_streams(streams)
             os.setsid()
 
-            for limit, value in self.rlimits.items():
-                res = getattr(resource, 'RLIMIT_%s' % limit.upper(), None)
-                if res is None:
-                    raise ValueError('unknown rlimit "%s"' % limit)
-                # TODO(petef): support hard/soft limits
-                resource.setrlimit(res, (value, value))
+            if resource:
+                for limit, value in self.rlimits.items():
+                    res = getattr(
+                        resource, 'RLIMIT_%s' % limit.upper(), None
+                    )
+                    if res is None:
+                        raise ValueError('unknown rlimit "%s"' % limit)
+
+                    # TODO(petef): support hard/soft limits
+
+                    # for the NOFILE limit, if we fail to set an unlimited
+                    # value then check the existing hard limit because we
+                    # probably can't bypass it due to a kernel limit - so just
+                    # assume that the caller means they want to use the kernel
+                    # limit when they pass the unlimited value. This is better
+                    # than failing to start the process and forcing the caller
+                    # to always be aware of what the kernel configuration is.
+                    # If they do pass in a real limit value, then we'll just
+                    # raise the failure as they should know that their
+                    # expectations couldn't be met.
+                    # TODO - we can't log here as this occurs in the child
+                    # process after the fork but it would be very good to
+                    # notify the admin of the situation somehow.
+                    retry = False
+                    try:
+                        resource.setrlimit(res, (value, value))
+                    except ValueError:
+                        if res == resource.RLIMIT_NOFILE and \
+                                value == resource.RLIM_INFINITY:
+                            _soft, value = resource.getrlimit(res)
+                            retry = True
+                        else:
+                            raise
+                    if retry:
+                        resource.setrlimit(res, (value, value))
 
             if self.gid:
                 try:
@@ -286,6 +330,12 @@ class Process(object):
             if self.uid:
                 os.setuid(self.uid)
 
+        if IS_WINDOWS:
+            # On Windows we can't use a pre-exec function
+            preexec_fn = None
+        else:
+            preexec_fn = preexec
+
         extra = {}
         if self.pipe_stdout:
             extra['stdout'] = PIPE
@@ -301,8 +351,6 @@ class Process(object):
         # let go of sockets created only for self._worker to inherit
         self._sockets = []
 
-        self.started = time.time()
-
     def format_args(self, sockets_fds=None):
         """ It's possible to use environment variables and some other variables
         that are available in this context, when spawning the processes.
@@ -331,8 +379,8 @@ class Process(object):
 
         if '$WID' in cmd or (self.args and '$WID' in self.args):
             msg = "Using $WID in the command is deprecated. You should use "\
-                  "the python string format instead. In you case, this means "\
-                  "replacing the $WID in your command by $(WID)."
+                  "the python string format instead. In your case, this "\
+                  "means replacing the $WID in your command by $(WID)."
 
             warnings.warn(msg, DeprecationWarning)
             self.cmd = cmd.replace('$WID', str(self.wid))
@@ -344,16 +392,16 @@ class Process(object):
             else:
                 args = [bytestring(replace_gnu_args(arg, **format_kwargs))
                         for arg in self.args]
-            args = shlex.split(bytestring(cmd)) + args
+            args = shlex.split(bytestring(cmd), posix=not IS_WINDOWS) + args
         else:
-            args = shlex.split(bytestring(cmd))
+            args = shlex.split(bytestring(cmd), posix=not IS_WINDOWS)
 
         if self.shell:
             # subprocess.Popen(shell=True) implies that 1st arg is the
             # requested command, remaining args are applied to sh.
             args = [' '.join(quote(arg) for arg in args)]
             shell_args = format_kwargs.get('shell_args', None)
-            if shell_args and is_win():
+            if shell_args and IS_WINDOWS:
                 logger.warn("shell_args won't apply for "
                             "windows platforms: %s", shell_args)
             elif isinstance(shell_args, string_types):
@@ -401,16 +449,33 @@ class Process(object):
         """
         try:
             try:
-                if self._worker.poll() is None:
-                    return self._worker.terminate()
+                if self.is_alive():
+                    try:
+                        return self._worker.terminate()
+                    except AccessDenied:
+                        # It can happen on Windows if the process
+                        # dies after poll returns (unlikely)
+                        pass
             finally:
-                if self._worker.stderr is not None:
-                    self._worker.stderr.close()
-                if self._worker.stdout is not None:
-                    self._worker.stdout.close()
+                self.close_output_channels()
         except NoSuchProcess:
             pass
 
+    def close_output_channels(self):
+        if self._worker.stderr is not None:
+            self._worker.stderr.close()
+        if self._worker.stdout is not None:
+            self._worker.stdout.close()
+
+    def wait(self, timeout=None):
+        """
+        Wait for the process to terminate, in the fashion
+        of waitpid.
+
+        Accepts a timeout in seconds.
+        """
+        self._worker.wait(timeout)
+
     def age(self):
         """Return the age of the process in seconds."""
         return time.time() - self.started
@@ -466,9 +531,9 @@ class Process(object):
             raise NoSuchProcess(pid)
 
     @debuglog
-    def send_signal_children(self, signum):
+    def send_signal_children(self, signum, recursive=False):
         """Send signal *signum* to all children."""
-        for child in get_children(self._worker):
+        for child in get_children(self._worker, recursive):
             try:
                 child.send_signal(signum)
             except OSError as e:
