@@ -250,8 +250,7 @@ def main():
                     subtopic = 'all'
 
                 # Clean pids that have not been updated recently
-                valid_pid = lambda p: p.isdigit() and p in watchers[watcher]
-                for pid in filter(valid_pid, watchers[watcher]):
+                for pid in tuple(p for p in watchers[watcher] if p.isdigit()):
                     if (last_refresh_for_pid[pid] <
                             time.time() - int(args.process_timeout)):
                         del watchers[watcher][pid]
