@@ -11,8 +11,10 @@ from circus.py3compat import s, PY2
 
 class _FileStreamBase(object):
     """Base class for all file writer handler classes"""
-    # You may want to use another now method (not naive or a mock).
+    # You may want to use another now or fromtimestamp method
+    # (not naive or a mock).
     now = datetime.now
+    fromtimestamp = datetime.fromtimestamp
 
     def __init__(self, filename, time_format):
         if filename is None:
@@ -26,6 +28,10 @@ class _FileStreamBase(object):
     def _open(self):
         return open(self._filename, 'a+')
 
+    def open(self):
+        if self._file.closed:
+            self._file = self._open()
+
     def close(self):
         self._file.close()
 
@@ -35,7 +41,11 @@ class _FileStreamBase(object):
 
         # If we want to prefix the stream with the current datetime
         if self._time_format is not None:
-            time = self.now().strftime(self._time_format)
+            if 'timestamp' in data:
+                time = self.fromtimestamp(data['timestamp'])
+            else:
+                time = self.now()
+            time = time.strftime(self._time_format)
             prefix = '{time} [{pid}] | '.format(time=time, pid=data['pid'])
             file_data = prefix + file_data.rstrip('\n')
             file_data = file_data.replace('\n', '\n' + prefix)
@@ -50,6 +60,8 @@ class _FileStreamBase(object):
                 file_data = file_data.encode('latin-1', errors='replace')
                 file_data = file_data.decode('latin-1')
                 self._file.write(file_data)
+            else:
+                raise
 
         self._file.flush()
 
