@@ -32,8 +32,8 @@ import tornado
 from circus import get_arbiter
 from circus.util import DEFAULT_ENDPOINT_DEALER, DEFAULT_ENDPOINT_SUB
 from circus.util import tornado_sleep, ConflictError
+from circus.util import IS_WINDOWS
 from circus.client import AsyncCircusClient, make_message
-from circus.stream import QueueStream
 
 ioloop.install()
 if 'ASYNC_TEST_TIMEOUT' not in os.environ:
@@ -49,41 +49,12 @@ class EasyTestSuite(TestSuite):
             pass
 
 
-def resolve_name(name):
-    ret = None
-    parts = name.split('.')
-    cursor = len(parts)
-    module_name = parts[:cursor]
-    last_exc = None
+PYTHON = sys.executable
 
-    while cursor > 0:
-        try:
-            ret = __import__('.'.join(module_name))
-            break
-        except ImportError as exc:
-            last_exc = exc
-            if cursor == 0:
-                raise
-            cursor -= 1
-            module_name = parts[:cursor]
-
-    for part in parts[1:]:
-        try:
-            ret = getattr(ret, part)
-        except AttributeError:
-            if last_exc is not None:
-                raise last_exc
-            raise ImportError(name)
-
-    if ret is None:
-        if last_exc is not None:
-            raise last_exc
-        raise ImportError(name)
-
-    return ret
-
-
-_CMD = sys.executable
+# Script used to sleep for a specified amount of seconds.
+# Should be used instead of the 'sleep' command for
+# compatibility
+SLEEP = PYTHON + " -c 'import time;time.sleep(%d)'"
 
 
 def get_ioloop():
@@ -227,11 +198,8 @@ class TestCircus(AsyncTestCase):
         return plugin
 
     @tornado.gen.coroutine
-    def start_arbiter(self, cmd='circus.tests.support.run_process',
+    def start_arbiter(self, cmd='support.run_process',
                       stdout_stream=None, debug=True, **kw):
-        if stdout_stream is None:
-            self.stream = QueueStream()
-            stdout_stream = {'stream': self.stream}
         testfile, arbiter = self._create_circus(
             cmd, stdout_stream=stdout_stream,
             debug=debug, async=True, **kw)
@@ -283,12 +251,12 @@ class TestCircus(AsyncTestCase):
     @classmethod
     def _create_circus(cls, callable_path, plugins=None, stats=False,
                        async=False, arbiter_kw=None, **kw):
-        resolve_name(callable_path)   # used to check the callable
         fd, testfile = mkstemp()
         os.close(fd)
-        wdir = os.path.dirname(__file__)
-        args = ['generic.py', callable_path, testfile]
-        worker = {'cmd': _CMD, 'args': args, 'working_dir': wdir,
+        wdir = os.path.dirname(os.path.dirname(os.path.dirname(
+            os.path.realpath(__file__))))
+        args = ['circus/tests/generic.py', callable_path, testfile]
+        worker = {'cmd': PYTHON, 'args': args, 'working_dir': wdir,
                   'name': 'test', 'graceful_timeout': 2}
         worker.update(kw)
         if not arbiter_kw:
@@ -355,11 +323,20 @@ class Process(object):
 
     def __init__(self, testfile):
         self.testfile = testfile
+
         # init signal handling
-        signal.signal(signal.SIGQUIT, self.handle_quit)
-        signal.signal(signal.SIGTERM, self.handle_quit)
-        signal.signal(signal.SIGINT, self.handle_quit)
-        signal.signal(signal.SIGCHLD, self.handle_chld)
+        if IS_WINDOWS:
+            signal.signal(signal.SIGABRT, self.handle_quit)
+            signal.signal(signal.SIGTERM, self.handle_quit)
+            signal.signal(signal.SIGINT, self.handle_quit)
+            signal.signal(signal.SIGILL, self.handle_quit)
+            signal.signal(signal.SIGBREAK, self.handle_quit)
+        else:
+            signal.signal(signal.SIGQUIT, self.handle_quit)
+            signal.signal(signal.SIGTERM, self.handle_quit)
+            signal.signal(signal.SIGINT, self.handle_quit)
+            signal.signal(signal.SIGCHLD, self.handle_chld)
+
         self.alive = True
 
     def _write(self, msg):
@@ -588,3 +565,7 @@ class MagicMockFuture(mock.MagicMock, tornado.concurrent.Future):
 
     def set_exception(self, exception):
         pass
+
+    def __del__(self):
+        # Don't try to print non-consumed exceptions
+        pass
