@@ -1,10 +1,11 @@
 import os
+import signal
 import socket
-import sys
 import tornado
 from tempfile import mkstemp
 from time import time
 import zmq.utils.jsonapi as json
+import mock
 try:
     from urllib.parse import urlparse
 except ImportError:
@@ -13,8 +14,9 @@ except ImportError:
 from circus.arbiter import Arbiter
 from circus.client import CircusClient
 from circus.plugins import CircusPlugin
-from circus.tests.support import TestCircus, async_poll_for, truncate_file
-from circus.tests.support import EasyTestSuite, skipIf, get_ioloop
+from circus.tests.support import (TestCircus, async_poll_for, truncate_file,
+                                  EasyTestSuite, skipIf, get_ioloop, SLEEP,
+                                  PYTHON)
 from circus.util import (DEFAULT_ENDPOINT_DEALER, DEFAULT_ENDPOINT_MULTICAST,
                          DEFAULT_ENDPOINT_SUB)
 from circus.watcher import Watcher
@@ -110,14 +112,14 @@ class TestTrainer(TestCircus):
         fd, testfile = mkstemp()
         os.close(fd)
         cmd = '%s %s %s %s' % (
-            sys.executable, _GENERIC,
+            PYTHON, _GENERIC,
             'circus.tests.support.run_process',
             testfile)
 
         return cmd
 
     def _get_cmd_args(self):
-        cmd = sys.executable
+        cmd = PYTHON
         args = [_GENERIC, 'circus.tests.support.run_process']
         return cmd, args
 
@@ -258,6 +260,28 @@ class TestTrainer(TestCircus):
         self.assertEqual(resp.get("numwatchers"), before - 1)
         yield self.stop_arbiter()
 
+    @tornado.testing.gen_test
+    def test_rm_watcher_nostop(self):
+        # start watcher, save off the pids for the watcher processes we
+        # started, stop the watcher without stopping processes, and validate
+        # the processes are still running, then kill the processes
+        yield self.start_arbiter(graceful_timeout=0)
+        name = 'test_rm_watcher_nostop'
+        yield self._call("add", name=name, cmd=self._get_cmd(), start=True,
+                         options=self._get_options())
+        resp = yield self._call("list", name=name)
+        pids = resp.get('pids')
+        self.assertEqual(len(pids), 1)
+        yield self._call("rm", name=name, nostop=True)
+        try:
+            pid = pids[0]
+            os.kill(pid, 0)
+            os.kill(pid, signal.SIGTERM)
+            os.waitpid(pid, 0)
+        except OSError:
+            self.assertFalse(True, "process was incorrectly killed")
+        yield self.stop_arbiter()
+
     @tornado.testing.gen_test
     def _test_stop(self):
         resp = yield self._call("quit")
@@ -291,6 +315,27 @@ class TestTrainer(TestCircus):
         self.assertNotEqual(processes1, processes2)
         yield self.stop_arbiter()
 
+    @tornado.testing.gen_test
+    def test_reload_uppercase(self):
+        yield self.start_arbiter(graceful_timeout=0)
+        name = 'test_RELOAD'
+        yield self._call("add", name=name, cmd=self._get_cmd(),
+                         start=True, options=self._get_options())
+
+        resp = yield self._call("list", name=name)
+        processes1 = resp.get('pids')
+
+        truncate_file(self.test_file)  # clean slate
+
+        yield self._call("reload")
+        self.assertTrue(async_poll_for(self.test_file, 'START'))  # restarted
+
+        resp = yield self._call("list", name=name)
+        processes2 = resp.get('pids')
+
+        self.assertNotEqual(processes1, processes2)
+        yield self.stop_arbiter()
+
     @tornado.testing.gen_test
     def test_reload_sequential(self):
         yield self.start_arbiter(graceful_timeout=0)
@@ -536,7 +581,7 @@ class TestTrainer(TestCircus):
 
         watcher_mod.tornado_sleep = _sleep
 
-        watcher = MockWatcher(name='foo', cmd='sleep 1', priority=1)
+        watcher = MockWatcher(name='foo', cmd=SLEEP % 1, priority=1)
         yield self.arbiter.start_watcher(watcher)
 
         self.assertTrue(called, [self.arbiter.warmup_delay])
@@ -563,6 +608,37 @@ class TestArbiter(TestCircus):
     Unit tests for the arbiter class to codify requirements within
     behavior.
     """
+    def test_start_with_callback(self):
+        controller = "tcp://127.0.0.1:%d" % get_available_port()
+        sub = "tcp://127.0.0.1:%d" % get_available_port()
+        arbiter = Arbiter([], controller, sub, check_delay=-1)
+
+        callee = mock.MagicMock()
+
+        def callback(*args):
+            callee()
+            arbiter.stop()
+
+        arbiter.start(cb=callback)
+
+        self.assertEqual(callee.call_count, 1)
+
+    @tornado.testing.gen_test
+    def test_start_with_callback_and_given_loop(self):
+        controller = "tcp://127.0.0.1:%d" % get_available_port()
+        sub = "tcp://127.0.0.1:%d" % get_available_port()
+        arbiter = Arbiter([], controller, sub, check_delay=-1,
+                          loop=get_ioloop())
+
+        callback = mock.MagicMock()
+
+        try:
+            yield arbiter.start(cb=callback)
+        finally:
+            yield arbiter.stop()
+
+        self.assertEqual(callback.call_count, 0)
+
     @tornado.testing.gen_test
     def test_start_watcher(self):
         watcher = MockWatcher(name='foo', cmd='serve', priority=1)
@@ -583,7 +659,7 @@ class TestArbiter(TestCircus):
         sub = "tcp://127.0.0.1:%d" % get_available_port()
         arbiter = Arbiter([], controller, sub, loop=get_ioloop(),
                           check_delay=-1)
-        arbiter.add_watcher('foo', 'sleep 5')
+        arbiter.add_watcher('foo', SLEEP % 5)
         try:
             yield arbiter.start()
             self.assertEqual(arbiter.watchers[0].status(), 'active')
@@ -595,7 +671,7 @@ class TestArbiter(TestCircus):
         arbiter = Arbiter([], DEFAULT_ENDPOINT_DEALER, DEFAULT_ENDPOINT_SUB,
                           loop=get_ioloop(),
                           check_delay=-1)
-        arbiter.add_watcher('foo', 'sleep 5', autostart=False)
+        arbiter.add_watcher('foo', SLEEP % 5, autostart=False)
         try:
             yield arbiter.start()
             self.assertEqual(arbiter.watchers[0].status(), 'stopped')
