@@ -1,5 +1,5 @@
 from circus.tests.test_command_incrproc import FakeArbiter
-from circus.tests.support import TestCircus
+from circus.tests.support import TestCircus, EasyTestSuite
 from circus.commands.decrproc import DecrProcess
 
 
@@ -13,3 +13,5 @@ class DecrProcTest(TestCircus):
         props = cmd.message('dummy')['properties']
         cmd.execute(arbiter, props)
         self.assertEqual(arbiter.watchers[0].nb, 0)
+
+test_suite = EasyTestSuite(__name__)
