@@ -1,4 +1,4 @@
-from circus.tests.support import TestCircus
+from circus.tests.support import TestCircus, EasyTestSuite
 from circus.commands.incrproc import IncrProc
 
 
@@ -21,16 +21,28 @@ class FakeWatcher(object):
         self.nb -= nb
 
 
+class FakeLoop(object):
+    def add_callback(self, function):
+        function()
+
+
 class FakeArbiter(object):
 
     watcher_class = FakeWatcher
 
     def __init__(self):
         self.watchers = [self.watcher_class()]
+        self.loop = FakeLoop()
 
     def get_watcher(self, name):
         return self.watchers[0]
 
+    def stop_watchers(self, **options):
+        self.watchers[:] = []
+
+    def stop(self, **options):
+        self.stop_watchers(**options)
+
 
 class IncrProcTest(TestCircus):
 
@@ -40,8 +52,7 @@ class IncrProcTest(TestCircus):
         self.assertTrue(message['properties'], {'name': 'dummy'})
 
         message = cmd.message('dummy', 3)
-        props = message['properties'].items()
-        props.sort()
+        props = sorted(message['properties'].items())
         self.assertEqual(props, [('name', 'dummy'), ('nb', 3)])
 
     def test_incr_proc(self):
@@ -52,3 +63,5 @@ class IncrProcTest(TestCircus):
         props = cmd.message('dummy', 3)['properties']
         cmd.execute(arbiter, props)
         self.assertEqual(arbiter.watchers[0].nb, size_before + 3)
+
+test_suite = EasyTestSuite(__name__)
