@@ -8,7 +8,7 @@ from circus.config import get_config
 from circus.watcher import Watcher
 from circus.process import Process
 from circus.sockets import CircusSocket
-from circus.tests.support import TestCase, EasyTestSuite
+from circus.tests.support import TestCase, EasyTestSuite, IS_WINDOWS
 from circus.util import replace_gnu_args
 from circus.py3compat import PY2
 
@@ -42,6 +42,7 @@ _CONF = {
     'issue651': os.path.join(CONFIG_DIR, 'issue651.ini'),
     'issue665': os.path.join(CONFIG_DIR, 'issue665.ini'),
     'issue680': os.path.join(CONFIG_DIR, 'issue680.ini'),
+    'virtualenv': os.path.join(CONFIG_DIR, 'virtualenv.ini')
 }
 
 
@@ -60,7 +61,7 @@ class TestConfig(TestCase):
 
     def test_issue310(self):
         '''
-        https://github.com/mozilla-services/circus/pull/310
+        https://github.com/circus-tent/circus/pull/310
 
         Allow $(circus.sockets.name) to be used in args.
         '''
@@ -69,7 +70,13 @@ class TestConfig(TestCase):
         socket = CircusSocket.load_from_config(conf['sockets'][0])
         try:
             watcher.initialize(None, {'web': socket}, None)
-            process = Process(watcher._nextwid, watcher.cmd,
+
+            if IS_WINDOWS:
+                # We can't close the sockets on Windows as we
+                # are redirecting stdout
+                watcher.use_sockets = True
+
+            process = Process('test', watcher._nextwid, watcher.cmd,
                               args=watcher.args,
                               working_dir=watcher.working_dir,
                               shell=watcher.shell, uid=watcher.uid,
@@ -95,7 +102,7 @@ class TestConfig(TestCase):
 
     def test_issues665(self):
         '''
-        https://github.com/mozilla-services/circus/pull/665
+        https://github.com/circus-tent/circus/pull/665
 
         Ensure args formatting when shell = True.
         '''
@@ -103,7 +110,12 @@ class TestConfig(TestCase):
 
         def load(watcher_conf):
             watcher = Watcher.load_from_config(watcher_conf.copy())
-            process = Process(watcher._nextwid, watcher.cmd,
+
+            # Make sure we don't close the sockets as we will be
+            # launching the Watcher with IS_WINDOWS=True
+            watcher.use_sockets = True
+
+            process = Process('test', watcher._nextwid, watcher.cmd,
                               args=watcher.args,
                               working_dir=watcher.working_dir,
                               shell=watcher.shell, uid=watcher.uid,
@@ -115,11 +127,10 @@ class TestConfig(TestCase):
             return process.format_args()
 
         import circus.process
-        is_win = circus.process.is_win
 
         try:
             # force nix
-            circus.process.is_win = lambda: False
+            circus.process.IS_WINDOWS = False
 
             # without shell_args
             with patch.object(logger, 'warn') as mock_logger_warn:
@@ -141,7 +152,7 @@ class TestConfig(TestCase):
                 self.assertTrue(mock_logger_warn.called)
 
             # force win
-            circus.process.is_win = lambda: True
+            circus.process.IS_WINDOWS = True
 
             # without shell_args
             with patch.object(logger, 'warn') as mock_logger_warn:
@@ -161,7 +172,7 @@ class TestConfig(TestCase):
                 self.assertEqual(formatted_args, ['foo', '--fd'])
                 self.assertTrue(mock_logger_warn.called)
         finally:
-            circus.process.is_win = is_win
+            circus.process.IS_WINDOWS = IS_WINDOWS
 
     def test_include_wildcards(self):
         conf = get_config(_CONF['include'])
@@ -175,7 +186,7 @@ class TestConfig(TestCase):
 
     @patch.object(logger, 'warn')
     def test_empty_include(self, mock_logger_warn):
-        """https://github.com/mozilla-services/circus/pull/473"""
+        """https://github.com/circus-tent/circus/pull/473"""
         try:
             get_config(_CONF['empty_include'])
         except:
@@ -282,7 +293,7 @@ class TestConfig(TestCase):
 
     def test_expand_vars(self):
         '''
-        https://github.com/mozilla-services/circus/pull/554
+        https://github.com/circus-tent/circus/pull/554
         '''
         conf = get_config(_CONF['expand_vars'])
         watcher = conf['watchers'][0]
@@ -355,5 +366,11 @@ class TestConfig(TestCase):
         conf = get_config(_CONF['issue651'])
         self.assertEqual(conf['check_delay'], 10.5)
 
+    def test_virtualenv(self):
+        conf = get_config(_CONF['virtualenv'])
+        watcher = conf['watchers'][0]
+        self.assertEqual(watcher['virtualenv'], "/tmp/.virtualenvs/test")
+        self.assertEqual(watcher['virtualenv_py_ver'], "3.3")
+
 
 test_suite = EasyTestSuite(__name__)
