@@ -3,12 +3,12 @@ import os
 import subprocess
 
 from circus.pidfile import Pidfile
-from circus.tests.support import TestCase, EasyTestSuite
+from circus.tests.support import TestCase, EasyTestSuite, SLEEP
 
 
 class TestPidfile(TestCase):
     def test_pidfile(self):
-        proc = subprocess.Popen('sleep 120', shell=True)
+        proc = subprocess.Popen(SLEEP % 120, shell=True)
         fd, path = tempfile.mkstemp()
         os.close(fd)
 
