@@ -19,6 +19,13 @@ class TestCollector(TestCase):
         self.fds = []
         self.pids = {}
 
+    def tearDown(self):
+        for sock, _, _ in self.socks:
+            sock.close()
+
+        for sock in self.clients:
+            sock.close()
+
     def _get_streamer(self):
         for i in range(10):
             sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
@@ -60,19 +67,20 @@ class TestCollector(TestCase):
 
             def __init__(this, streamer):
                 Thread.__init__(this)
-                this.streamer = self.streamer
+                this.streamer = streamer
                 this.loop = ioloop.IOLoop()
                 this.daemon = True
 
-            def run(this):
+            def run(self):
                 collector = collector_class(
                     self.streamer, 'sockets', callback_time=0.1,
-                    io_loop=this.loop)
+                    io_loop=self.loop)
                 collector.start()
-                this.loop.start()
+                self.loop.start()
 
             def stop(self):
-                self.loop.stop()
+                self.loop.add_callback(self.loop.stop)
+                self.loop.add_callback(self.loop.close)
 
         return Collector(self.streamer)
 
@@ -172,7 +180,7 @@ class TestCollector(TestCase):
         for i in range(10):
             for client in self.clients:
                 client.send(b'ok')
-                #client.recv(2)
+                # client.recv(2)
 
         # stopping
         collector.stop()
