@@ -1,15 +1,10 @@
 import os
 import tempfile
-import time
 
 import mock
-from zmq.eventloop import ioloop
-import zmq.utils.jsonapi as json
 
-from circus.stats.collector import SocketStatsCollector
 from circus.tests.support import TestCircus, EasyTestSuite
 from circus.stats.streamer import StatsStreamer
-from circus import util
 from circus import client
 
 
@@ -19,7 +14,6 @@ class _StatsStreamer(StatsStreamer):
 
     def handle_recv(self, data):
         self.msgs.append(data)
-        super(_StatsStreamer, self).handle_recv(data)
 
 
 class FakeStreamer(StatsStreamer):
@@ -51,54 +45,13 @@ class TestStatsStreamer(TestCircus):
         elif what == 'listsockets':
             return {'status': 'ok',
                     'sockets': [{'path': self._unix,
-                                'fd': 5,
-                                'name': 'XXXX',
-                                'backlog': 2048}],
+                                 'fd': 5,
+                                 'name': 'XXXX',
+                                 'backlog': 2048}],
                     'time': 1369647058.967524}
 
         raise NotImplementedError(cmd)
 
-    def test_socketstats(self):
-
-        endpoint = util.DEFAULT_ENDPOINT_DEALER
-        pubsub = util.DEFAULT_ENDPOINT_SUB
-        statspoint = util.DEFAULT_ENDPOINT_STATS
-
-        loop = ioloop.IOLoop()
-        streamer = _StatsStreamer(endpoint, pubsub, statspoint,
-                                  loop=loop)
-
-        # now the stats collector
-        self._collector = SocketStatsCollector(streamer, 'sockets',
-                                               callback_time=0.1,
-                                               io_loop=loop)
-
-        self._collector.start()
-        loop.add_callback(streamer._init)
-
-        # events
-        def _events():
-            msg = 'one.spawn.two', json.dumps({'process_pid': 187})
-            for i in range(5):
-                streamer.handle_recv(msg)
-
-        deadline = time.time() + 0.5
-        loop.add_timeout(deadline, _events)
-
-        def _stop():
-            self._collector.stop()
-            streamer.stop()
-
-        deadline = time.time() + 0.5
-        loop.add_timeout(deadline, _stop)
-        streamer.start()
-
-        # let's see what we got
-        try:
-            self.assertTrue(len(streamer.msgs) > 1)
-        finally:
-            streamer.stop()
-
     def test_get_pids_circus(self):
         streamer = FakeStreamer()
         streamer.circus_pids = {1234: 'circus-top', 1235: 'circusd'}
