@@ -9,10 +9,15 @@ import sys
 import time
 import traceback
 import json
+import struct
 try:
     import yaml
 except ImportError:
     yaml = None  # NOQA
+try:
+    import papa
+except ImportError:
+    papa = None  # NOQA
 try:
     import pwd
     import grp
@@ -97,6 +102,8 @@ LOG_DATE_SYSLOG_FMT = r"%b %d %H:%M:%S"
 _SYMBOLS = ('K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y')
 _all_signals = {}
 
+IS_WINDOWS = os.name == 'nt'
+
 
 def get_working_dir():
     """Returns current path, try to use PWD env first.
@@ -259,7 +266,9 @@ def get_info(process=None, interval=0, with_childs=False):
     raw_cmdline = get_cmdline(process)
 
     try:
-        cmdline = os.path.basename(shlex.split(raw_cmdline[0])[0])
+        cmdline = os.path.basename(
+            shlex.split(raw_cmdline[0], posix=not IS_WINDOWS)[0]
+        )
     except (AccessDenied, IndexError):
         cmdline = "N/A"
 
@@ -291,6 +300,8 @@ FALSY_STRINGS = ('no', 'false', 'off', '0')
 def to_bool(s):
     if isinstance(s, bool):
         return s
+    if s is None:
+        return False
 
     if s.lower().strip() in TRUTHY_STRINGS:
         return True
@@ -678,8 +689,14 @@ def configure_logger(logger, level='INFO', output="-", loggerconfig=None,
             handler = logging.handlers.SysLogHandler(
                 address=address, facility=facility)
         else:
-            handler = logging.handlers.WatchedFileHandler(output)
-            close_on_exec(handler.stream.fileno())
+            if not IS_WINDOWS:
+                handler = logging.handlers.WatchedFileHandler(output)
+                close_on_exec(handler.stream.fileno())
+            else:
+                # WatchedFileHandler is not supported on Windows,
+                # but a FileHandler should be a good drop-in replacement
+                # as log files are locked
+                handler = logging.FileHandler(output)
         formatter = logging.Formatter(fmt=LOG_FMT, datefmt=datefmt)
         handler.setFormatter(formatter)
         root_logger.handlers = [handler]
@@ -828,11 +845,12 @@ def get_connection(socket, endpoint, ssh_server=None, ssh_keyfile=None):
                               "Paramiko.  You need to install Paramiko")
 
 
-def load_virtualenv(watcher):
+def load_virtualenv(watcher, py_ver=None):
     if not watcher.copy_env:
         raise ValueError('copy_env must be True to to use virtualenv')
 
-    py_ver = sys.version.split()[0][:3]
+    if not py_ver:
+        py_ver = sys.version.split()[0][:3]
 
     # XXX Posix scheme - need to add others
     sitedir = os.path.join(watcher.virtualenv, 'lib', 'python' + py_ver,
@@ -924,7 +942,10 @@ def create_udp_socket(mcast_addr, mcast_port):
             # see #699
             pass
     # Put packet ttl to max
-    sock.setsockopt(socket.IPPROTO_IP, socket.IP_MULTICAST_TTL, 255)
+    # The following ttl fix is to make this work on SunOS and BSD systems.
+    # Ref : Issue #875
+    ttl = struct.pack('B', 255)
+    sock.setsockopt(socket.IPPROTO_IP, socket.IP_MULTICAST_TTL, ttl)
     # Register socket to multicast group
     sock.setsockopt(socket.IPPROTO_IP, socket.IP_ADD_MEMBERSHIP,
                     socket.inet_aton(mcast_addr) + socket.inet_aton(any_addr))
@@ -1019,10 +1040,11 @@ class TransformableFuture(concurrent.Future):
 
     _upstream_future = None
     _upstream_callback = None
-    _transform_function = lambda x: x
     _result = None
     _exception = None
 
+    def _transform_function(x): return x
+
     def set_transform_function(self, fn):
         self._transform_function = fn
 
@@ -1062,8 +1084,3 @@ def check_future_exception_and_log(future):
                 exc_info = future.exc_info()
                 traceback.print_tb(exc_info[2])
             return exception
-
-
-def is_win():
-    """checks if platform is Windows"""
-    return sys.platform == "win32"
