@@ -5,23 +5,28 @@ import signal
 import time
 import sys
 from random import randint
+
 try:
     from itertools import zip_longest as izip_longest
 except ImportError:
+    # noinspection PyUnresolvedReferences
     from itertools import izip_longest  # NOQA
 import site
 from tornado import gen
 
-from psutil import NoSuchProcess
+from psutil import NoSuchProcess, TimeoutExpired
 import zmq.utils.jsonapi as json
 from zmq.eventloop import ioloop
 
 from circus.process import Process, DEAD_OR_ZOMBIE, UNEXISTING
+from circus.papa_process_proxy import PapaProcessProxy
 from circus import logger
 from circus import util
-from circus.stream import get_pipe_redirector, get_stream
-from circus.util import parse_env_dict, resolve_name, tornado_sleep
-from circus.py3compat import bytestring, is_callable, b
+from circus.stream import get_stream, Redirector
+from circus.stream.papa_redirector import PapaRedirector
+from circus.util import parse_env_dict, resolve_name, tornado_sleep, IS_WINDOWS
+from circus.util import papa
+from circus.py3compat import bytestring, is_callable, b, PY2
 
 
 class Watcher(object):
@@ -94,6 +99,8 @@ class Watcher(object):
       - **name** - the stream name (*stderr* or *stdout*)
       - **data** - the data
 
+      This is not supported on Windows.
+
     - **stderr_stream**: a mapping that defines the stream for
       the process stderr. Defaults to None.
 
@@ -117,6 +124,8 @@ class Watcher(object):
       - **name** - the stream name (*stderr* or *stdout*)
       - **data** - the data
 
+      This is not supported on Windows.
+
     - **priority** -- integer that defines a priority for the watcher. When
       the Arbiter do some operations on all watchers, it will sort them
       with this field, from the bigger number to the smallest.
@@ -134,7 +143,9 @@ class Watcher(object):
       (default: False)
 
     - **copy_env** -- If True, the environment in which circus is running
-      run will be reproduced for the workers. (default: False)
+      run will be reproduced for the workers. This defaults to True on
+      Windows as you cannot run any executable without the **SYSTEMROOT**
+      variable. (default: False)
 
     - **copy_path** -- If True, circusd *sys.path* is sent to the
       process through *PYTHONPATH*. You must activate **copy_env** for
@@ -173,6 +184,9 @@ class Watcher(object):
 
     - **close_child_stderr**: If True, closes the stderr after the fork.
       default: False.
+
+    - **use_papa**: If True, use the papa process kernel for this process.
+      default: False.
     """
 
     def __init__(self, name, cmd, args=None, numprocesses=1, warmup_delay=0.,
@@ -186,7 +200,7 @@ class Watcher(object):
                  max_age_variance=30, hooks=None, respawn=True,
                  autostart=True, on_demand=False, virtualenv=None,
                  close_child_stdout=False, close_child_stderr=False,
-                 **options):
+                 virtualenv_py_ver=None, use_papa=False, **options):
         self.name = name
         self.use_sockets = use_sockets
         self.on_demand = on_demand
@@ -204,13 +218,14 @@ class Watcher(object):
         self.stderr_stream_conf = copy.copy(stderr_stream)
         self.stdout_stream = get_stream(self.stdout_stream_conf)
         self.stderr_stream = get_stream(self.stderr_stream_conf)
-        self.stdout_redirector = self.stderr_redirector = None
+        self.stream_redirector = None
         self.max_retry = max_retry
         self._options = options
         self.singleton = singleton
         self.copy_env = copy_env
         self.copy_path = copy_path
         self.virtualenv = virtualenv
+        self.virtualenv_py_ver = virtualenv_py_ver
         self.max_age = int(max_age)
         self.max_age_variance = int(max_age_variance)
         self.ignore_hook_failure = ['before_stop', 'after_stop',
@@ -221,22 +236,35 @@ class Watcher(object):
         self.autostart = autostart
         self.close_child_stdout = close_child_stdout
         self.close_child_stderr = close_child_stderr
+        self.use_papa = use_papa and papa is not None
         self.loop = loop or ioloop.IOLoop.instance()
 
         if singleton and self.numprocesses not in (0, 1):
             raise ValueError("Cannot have %d processes with a singleton "
                              " watcher" % self.numprocesses)
 
+        if IS_WINDOWS:
+            if self.stdout_stream or self.stderr_stream:
+                raise NotImplementedError("Streams are not supported"
+                                          " on Windows.")
+
+            if not copy_env and not env:
+                # Copy the env by default on Windows as we can't run any
+                # executable without some env variables
+                # Eventually, we could set only some required variables,
+                # such as SystemRoot
+                self.copy_env = True
+
         self.optnames = (("numprocesses", "warmup_delay", "working_dir",
                           "uid", "gid", "send_hup", "stop_signal",
                           "stop_children", "shell", "shell_args",
-                          "env", "max_retry", "cmd", "args",
+                          "env", "max_retry", "cmd", "args", "respawn",
                           "graceful_timeout", "executable", "use_sockets",
                           "priority", "copy_env", "singleton",
                           "stdout_stream_conf", "on_demand",
                           "stderr_stream_conf", "max_age", "max_age_variance",
-                          "close_child_stdout", "close_child_stderr")
-                         + tuple(options.keys()))
+                          "close_child_stdout", "close_child_stderr",
+                          "use_papa") + tuple(options.keys()))
 
         if not working_dir:
             # working dir hasn't been set
@@ -263,7 +291,7 @@ class Watcher(object):
             self.env = env
 
         if self.virtualenv:
-            util.load_virtualenv(self)
+            util.load_virtualenv(self, py_ver=virtualenv_py_ver)
 
         # load directories in PYTHONPATH if provided
         # so if a hook is there, it can be loaded
@@ -281,64 +309,64 @@ class Watcher(object):
         self.arbiter = None
         self.hooks = {}
         self._resolve_hooks(hooks)
+        self._found_wids = []
+
+        if self.use_papa:
+            with papa.Papa() as p:
+                base_name = 'circus.{0}.*'.format(name.lower())
+                running = p.list_processes(base_name)
+                self._found_wids = [int(proc_name[len(base_name) - 1:])
+                                    for proc_name in running]
 
     def _reload_hook(self, key, hook, ignore_error):
         hook_name = key.split('.')[-1]
         self._resolve_hook(hook_name, hook, ignore_error, reload_module=True)
 
+    @property
+    def _redirector_class(self):
+        return PapaRedirector if self.use_papa else Redirector
+
+    @property
+    def _process_class(self):
+        return PapaProcessProxy if self.use_papa else Process
+
     def _reload_stream(self, key, val):
         parts = key.split('.', 1)
 
-        action = 0
-        if parts[0] == 'stdout_stream':
-            old_stream = self.stdout_stream
+        stream_type = 'stdout' if parts[0] == 'stdout_stream' else 'stderr'
+        old_stream = self.stream_redirector.get_stream(stream_type) if\
+            self.stream_redirector else None
+        if stream_type == 'stdout':
             self.stdout_stream_conf[parts[1]] = val
-            self.stdout_stream = get_stream(self.stdout_stream_conf,
-                                            reload=True)
-            if self.stdout_redirector:
-                self.stdout_redirector.redirect = self.stdout_stream['stream']
-            else:
-                self.stdout_redirector = get_pipe_redirector(
-                    self.stdout_stream, loop=self.loop)
-                self.stdout_redirector.start()
-                action = 1
-
-            if old_stream and hasattr(old_stream['stream'], 'close'):
-                old_stream['stream'].close()
+            new_stream = get_stream(self.stdout_stream_conf, reload=True)
+            self.stdout_stream = new_stream
         else:
-            old_stream = self.stderr_stream
             self.stderr_stream_conf[parts[1]] = val
-            self.stderr_stream = get_stream(self.stderr_stream_conf,
-                                            reload=True)
-            if self.stderr_redirector:
-                self.stderr_redirector.redirect = self.stderr_stream['stream']
-            else:
-                self.stderr_redirector = get_pipe_redirector(
-                    self.stderr_stream, loop=self.loop)
-                self.stderr_redirector.start()
-                action = 1
+            new_stream = get_stream(self.stderr_stream_conf, reload=True)
+            self.stderr_stream = new_stream
 
-            if old_stream and hasattr(old_stream['stream'], 'close'):
-                old_stream['stream'].close()
+        if self.stream_redirector:
+            self.stream_redirector.change_stream(stream_type, new_stream)
+        else:
+            self.stream_redirector = self._redirector_class(
+                self.stdout_stream, self.stderr_stream, loop=self.loop)
 
-        return action
+        if old_stream:
+            if hasattr(old_stream, 'close'):
+                old_stream.close()
+            return 0
+
+        self.stream_redirector.start()
+        return 1
 
     def _create_redirectors(self):
-        if self.stdout_stream:
-            if self.stdout_redirector is not None:
-                self.stdout_redirector.stop()
-            self.stdout_redirector = get_pipe_redirector(
-                self.stdout_stream, loop=self.loop)
+        if self.stdout_stream or self.stderr_stream:
+            if self.stream_redirector:
+                self.stream_redirector.stop()
+            self.stream_redirector = self._redirector_class(
+                self.stdout_stream, self.stderr_stream, loop=self.loop)
         else:
-            self.stdout_redirector = None
-
-        if self.stderr_stream:
-            if self.stderr_redirector is not None:
-                self.stderr_redirector.stop()
-            self.stderr_redirector = get_pipe_redirector(
-                self.stderr_stream, loop=self.loop)
-        else:
-            self.stderr_redirector = None
+            self.stream_redirector = None
 
     def _resolve_hook(self, name, callable_or_name, ignore_failure,
                       reload_module=False):
@@ -401,50 +429,68 @@ class Watcher(object):
             return
         process = self.processes.pop(pid)
 
-        if status is None:
-            while True:
+        timeout = 0.001
+
+        while status is None:
+            if IS_WINDOWS:
+                try:
+                    # On Windows we can't use waitpid as it's blocking,
+                    # so we use psutils' wait
+                    status = process.wait(timeout=timeout)
+                except TimeoutExpired:
+                    continue
+            else:
                 try:
                     _, status = os.waitpid(pid, os.WNOHANG)
                 except OSError as e:
                     if e.errno == errno.EAGAIN:
-                        time.sleep(0.001)
+                        time.sleep(timeout)
                         continue
                     elif e.errno == errno.ECHILD:
-                        # nothing to do here, we do not have any child
-                        # process running
-                        # but we still need to send the "reap" signal.
-                        #
-                        # This can happen if poll() or wait() were called on
-                        # the underlying process.
-                        logger.debug('reaping already dead process %s [%s]',
-                                     pid, self.name)
-                        self.notify_event(
-                            "reap",
-                            {"process_pid": pid,
-                             "time": time.time(),
-                             "exit_code": process.returncode()})
-                        process.stop()
-                        return
+                        status = None
                     else:
                         raise
 
+            if status is None:
+                # nothing to do here, we do not have any child
+                # process running
+                # but we still need to send the "reap" signal.
+                #
+                # This can happen if poll() or wait() were called on
+                # the underlying process.
+                logger.debug('reaping already dead process %s [%s]',
+                             pid, self.name)
+                self.notify_event(
+                    "reap",
+                    {"process_pid": pid,
+                     "time": time.time(),
+                     "exit_code": process.returncode()})
+                process.stop()
+                return
+
         # get return code
-        if os.WIFSIGNALED(status):
-            # The Python Popen object returns <-signal> in it's returncode
-            # property if the process exited on a signal, so emulate that
-            # behavior here so that pubsub clients watching for reap can
-            # distinguish between an exit with a non-zero exit code and
-            # a signal'd exit. This is also consistent with the notify_event
-            # reap message above that uses the returncode function (that ends
-            # up calling Popen.returncode)
-            exit_code = -os.WTERMSIG(status)
-        # process exited using exit(2) system call; return the
-        # integer exit(2) system call has been called with
-        elif os.WIFEXITED(status):
-            exit_code = os.WEXITSTATUS(status)
+        if hasattr(os, 'WIFSIGNALED'):
+            exit_code = 0
+
+            if os.WIFSIGNALED(status):
+                # The Python Popen object returns <-signal> in it's returncode
+                # property if the process exited on a signal, so emulate that
+                # behavior here so that pubsub clients watching for reap can
+                # distinguish between an exit with a non-zero exit code and
+                # a signal'd exit. This is also consistent with the notify
+                # event reap message above that uses the returncode function
+                # (that ends up calling Popen.returncode)
+                exit_code = -os.WTERMSIG(status)
+            # process exited using exit(2) system call; return the
+            # integer exit(2) system call has been called with
+            elif os.WIFEXITED(status):
+                exit_code = os.WEXITSTATUS(status)
+            else:
+                # should never happen
+                raise RuntimeError("Unknown process exit status")
         else:
-            # should never happen
-            raise RuntimeError("Unknown process exit status")
+            # On Windows we don't have such distinction
+            exit_code = status
 
         # if the process is dead or a zombie try to definitely stop it.
         if process.status in (DEAD_OR_ZOMBIE, UNEXISTING):
@@ -510,9 +556,9 @@ class Watcher(object):
     @gen.coroutine
     @util.debuglog
     def remove_expired_processes(self):
-        max_age = self.max_age + randint(0, self.max_age_variance)
         expired_processes = [p for p in self.processes.values()
-                             if p.age() > max_age]
+                             if p.age() > (self.max_age + randint(0,
+                                           self.max_age_variance))]
         removes = yield [self.kill_process(x) for x in expired_processes]
         for i, process in enumerate(expired_processes):
             if removes[i]:
@@ -537,23 +583,32 @@ class Watcher(object):
         if self.pending_socket_event:
             self._status = "stopped"
             return
+        for i in self._found_wids:
+            self.spawn_process(i)
+            yield tornado_sleep(0)
+        self._found_wids = {}
+
         for i in range(self.numprocesses - len(self.processes)):
             res = self.spawn_process()
             if res is False:
                 yield self._stop()
                 break
-            yield tornado_sleep(self.warmup_delay)
+            delay = self.warmup_delay
+            if isinstance(res, float):
+                delay -= (time.time() - res)
+                if delay < 0:
+                    delay = 0
+            yield tornado_sleep(delay)
 
     def _get_sockets_fds(self):
         # XXX should be cached
         if self.sockets is None:
             return {}
-        fds = {}
-        for name, sock in self.sockets.items():
-            fds[name] = sock.fileno()
-        return fds
+        return dict((name, sock.fileno())
+                    for name, sock in self.sockets.items()
+                    if sock.use_papa == self.use_papa)
 
-    def spawn_process(self):
+    def spawn_process(self, recovery_wid=None):
         """Spawn process.
 
         Return True if ok, False if the watcher must be stopped
@@ -561,20 +616,27 @@ class Watcher(object):
         if self.is_stopped():
             return True
 
-        if not self.call_hook('before_spawn'):
+        if not recovery_wid and not self.call_hook('before_spawn'):
             return False
 
         cmd = util.replace_gnu_args(self.cmd, env=self.env)
         nb_tries = 0
 
+        # start the redirector now so we can catch any startup errors
+        if self.stream_redirector:
+            self.stream_redirector.start()
+
         while nb_tries < self.max_retry or self.max_retry == -1:
             process = None
-            pipe_stdout = self.stdout_redirector is not None
-            pipe_stderr = self.stderr_redirector is not None
+            pipe_stdout = self.stdout_stream is not None
+            pipe_stderr = self.stderr_stream is not None
 
+            # noinspection PyPep8Naming
+            ProcCls = self._process_class
             try:
-                process = Process(self._nextwid, cmd,
-                                  args=self.args, working_dir=self.working_dir,
+                process = ProcCls(self.name, recovery_wid or self._nextwid,
+                                  cmd, args=self.args,
+                                  working_dir=self.working_dir,
                                   shell=self.shell, uid=self.uid, gid=self.gid,
                                   env=self.env, rlimits=self.rlimits,
                                   executable=self.executable,
@@ -585,15 +647,8 @@ class Watcher(object):
                                   close_child_stderr=self.close_child_stderr)
 
                 # stream stderr/stdout if configured
-                if pipe_stdout and self.stdout_redirector is not None:
-                    self.stdout_redirector.add_redirection('stdout',
-                                                           process,
-                                                           process.stdout)
-
-                if pipe_stderr and self.stderr_redirector is not None:
-                    self.stderr_redirector.add_redirection('stderr',
-                                                           process,
-                                                           process.stderr)
+                if self.stream_redirector:
+                    self.stream_redirector.add_redirections(process)
 
                 self.processes[process.pid] = process
                 logger.debug('running %s process [pid %d]', self.name,
@@ -602,7 +657,10 @@ class Watcher(object):
                     self.kill_process(process)
                     del self.processes[process.pid]
                     return False
-            except OSError as e:
+
+            # catch ValueError as well, as a misconfigured rlimit setting could
+            # lead to bad infinite retries here
+            except (OSError, ValueError) as e:
                 logger.warning('error in %r: %s', self.name, str(e))
 
             if process is None:
@@ -610,8 +668,8 @@ class Watcher(object):
                 continue
             else:
                 self.notify_event("spawn", {"process_pid": process.pid,
-                                            "time": time.time()})
-                return True
+                                            "time": process.started})
+                return process.started
         return False
 
     @util.debuglog
@@ -644,14 +702,6 @@ class Watcher(object):
                 # already dead !
                 pass
 
-    def _process_remove_redirections(self, process):
-        """Remove process redirections
-        """
-        if self.stdout_redirector is not None and process.stdout is not None:
-            self.stdout_redirector.remove_redirection(process.stdout)
-        if self.stderr_redirector is not None and process.stderr is not None:
-            self.stderr_redirector.remove_redirection(process.stderr)
-
     @gen.coroutine
     @util.debuglog
     def kill_process(self, process):
@@ -673,14 +723,19 @@ class Watcher(object):
         process.stopping = True
         waited = 0
         while waited < self.graceful_timeout:
-            yield tornado_sleep(1)
-            waited += 1
             if not process.is_alive():
                 break
+            yield tornado_sleep(0.1)
+            waited += 0.1
         if waited >= self.graceful_timeout:
-            # We are not smart anymore
-            self.send_signal_process(process, signal.SIGKILL)
-        self._process_remove_redirections(process)
+            # On Windows we can't send a SIGKILL signal, but the
+            # process.stop function will terminate the process
+            # later anyway
+            if hasattr(signal, 'SIGKILL'):
+                # We are not smart anymore
+                self.send_signal_process(process, signal.SIGKILL)
+        if self.stream_redirector:
+            self.stream_redirector.remove_redirections(process)
         process.stopping = False
         process.stop()
         raise gen.Return(True)
@@ -699,11 +754,12 @@ class Watcher(object):
 
     @util.debuglog
     def send_signal(self, pid, signum):
+        is_sigkill = hasattr(signal, 'SIGKILL') and signum == signal.SIGKILL
         if pid in self.processes:
             process = self.processes[pid]
             hook_result = self.call_hook("before_signal",
                                          pid=pid, signum=signum)
-            if signum != signal.SIGKILL and not hook_result:
+            if not is_sigkill and not hook_result:
                 logger.debug("before_signal hook didn't return True "
                              "=> signal %i is not sent to %i" % (signum, pid))
             else:
@@ -724,11 +780,11 @@ class Watcher(object):
                 raise
 
     @util.debuglog
-    def send_signal_children(self, pid, signum):
+    def send_signal_children(self, pid, signum, recursive=False):
         """Send signal to all children.
         """
         process = self.processes[int(pid)]
-        process.send_signal_children(signum)
+        process.send_signal_children(signum, recursive)
 
     @util.debuglog
     def status(self):
@@ -758,43 +814,44 @@ class Watcher(object):
     @util.synchronized("watcher_stop")
     @gen.coroutine
     def stop(self):
-        yield self._stop()
+        # stop streams too since we are stopping the watcher completely
+        yield self._stop(True)
 
     @util.debuglog
     @gen.coroutine
-    def _stop(self, close_output_streams=False):
+    def _stop(self, close_output_streams=False, for_shutdown=False):
         if self.is_stopped():
             return
         self._status = "stopping"
-        logger.debug('stopping the %s watcher' % self.name)
-        logger.debug('gracefully stopping processes [%s] for %ss' % (
-                     self.name, self.graceful_timeout))
-        # We ignore the hook result
-        self.call_hook('before_stop')
-        yield self.kill_processes()
-        self.reap_processes()
+        skip = for_shutdown and self.use_papa
+        if not skip:
+            logger.debug('stopping the %s watcher' % self.name)
+            logger.debug('gracefully stopping processes [%s] for %ss' % (
+                         self.name, self.graceful_timeout))
+            # We ignore the hook result
+            self.call_hook('before_stop')
+            yield self.kill_processes()
+            self.reap_processes()
 
         # stop redirectors
-        if self.stdout_redirector is not None:
-            self.stdout_redirector.stop()
-            self.stdout_redirector = None
-        if self.stderr_redirector is not None:
-            self.stderr_redirector.stop()
-            self.stderr_redirector = None
+        if self.stream_redirector:
+            self.stream_redirector.stop()
+            self.stream_redirector = None
         if close_output_streams:
-            if self.stdout_stream and hasattr(self.stdout_stream['stream'],
-                                              'close'):
-                self.stdout_stream['stream'].close()
-            if self.stderr_stream and hasattr(self.stderr_stream['stream'],
-                                              'close'):
-                self.stderr_stream['stream'].close()
+            if self.stdout_stream and hasattr(self.stdout_stream, 'close'):
+                self.stdout_stream.close()
+            if self.stderr_stream and hasattr(self.stderr_stream, 'close'):
+                self.stderr_stream.close()
         # notify about the stop
-        if self.evpub_socket is not None:
-            self.notify_event("stop", {"time": time.time()})
-        self._status = "stopped"
-        # We ignore the hook result
-        self.call_hook('after_stop')
-        logger.info('%s stopped', self.name)
+        if skip:
+            logger.info('%s left running in papa', self.name)
+        else:
+            if self.evpub_socket is not None:
+                self.notify_event("stop", {"time": time.time()})
+            self._status = "stopped"
+            # We ignore the hook result
+            self.call_hook('after_stop')
+            logger.info('%s stopped', self.name)
 
     def get_active_processes(self):
         """return a list of pids of active processes (not already stopped)"""
@@ -865,12 +922,18 @@ class Watcher(object):
                 yield self.spawn_processes()
             return
 
-        if not self.call_hook('before_start'):
+        found_wids = len(self._found_wids)
+        if not self._found_wids and not self.call_hook('before_start'):
             logger.debug('Aborting startup')
             return
 
         self._status = "starting"
 
+        if self.stdout_stream and hasattr(self.stdout_stream, 'open'):
+            self.stdout_stream.open()
+        if self.stderr_stream and hasattr(self.stderr_stream, 'open'):
+            self.stderr_stream.open()
+
         self._create_redirectors()
         self.reap_processes()
         yield self.spawn_processes()
@@ -879,17 +942,15 @@ class Watcher(object):
         # probably prevented startup so give up
         if not self.processes or not self.call_hook('after_start'):
             logger.debug('Aborting startup')
-            yield self._stop()
+            # stop streams too since we are bailing on this watcher completely
+            yield self._stop(True)
             return
 
-        if self.stdout_redirector is not None:
-            self.stdout_redirector.start()
-
-        if self.stderr_redirector is not None:
-            self.stderr_redirector.start()
-
         self._status = "active"
-        logger.info('%s started' % self.name)
+        if found_wids:
+            logger.info('%s already running' % self.name)
+        else:
+            logger.info('%s started' % self.name)
         self.notify_event("start", {"time": time.time()})
 
     @util.synchronized("watcher_restart")
@@ -923,7 +984,7 @@ class Watcher(object):
     def _reload(self, graceful=True, sequential=False):
         """ reload
         """
-        if not(graceful) and sequential:
+        if not graceful and sequential:
             logger.warn("with graceful=False, sequential=True is ignored")
         if self.prereload_fn is not None:
             self.prereload_fn(self)
@@ -1020,7 +1081,12 @@ class Watcher(object):
             self.shell = val
             action = 1
         elif key == "env":
-            self.env = val
+            if PY2 and IS_WINDOWS:
+                # Windows on Python 2 does not accept Unicode values
+                # in env dictionary
+                self.env = dict((b(k), b(v)) for k, v in val.iteritems())
+            else:
+                self.env = val
             action = 1
         elif key == "cmd":
             self.cmd = val
